/*
 * The code for the Main Window 
 * Copyright (c) 1998 - Stig Erik Sand
 *
 * $Id: PrefDialog.cpp,v 1.12 1998/05/23 20:30:49 stig Exp $
 */

#include "PrefDialog.h"
#include "paintarea.H"
#include <stdlib.h>
#include <qfiledlg.h>
#include <qapp.h>

#define Inherited PrefDialogData
#define MYAPP_NAME "Macbeth"

PrefDialog::PrefDialog(QWidget* parent, const char* name) :
    Inherited( parent, name ) {

    setCaption( MYAPP_NAME );

    origdisp = dispOrigCheck->isChecked();
    dcheck = darkTestCheck->isChecked();
    colsubst = colSubstCheck->isChecked();
    contdisp = contDispCheck->isChecked();
    pcheck = printableCheck->isChecked();

    const char *newX = newXInput->text();
    const char *newY = newYInput->text();
    const char *newSX = splitXInput->text();
    const char *newSY = splitYInput->text();
    
    if (!newX || !strlen(newX)) tileSizeX=DEFAULT_TILEX;
    else tileSizeX=atoi(newX);
    if (!newY || !strlen(newY)) tileSizeY=DEFAULT_TILEY;
    else tileSizeY=atoi(newY);
    if (!newSX || !strlen(newSX)) splitX=PRINTABLE_WIDTH;
    else tileSizeX=atoi(newSX);

    if (!newSY || !strlen(newSY)) splitY=PRINTABLE_HEIGHT;
    else splitY=atoi(newSY);
   
    infilename = infileInput->text();
    if (!infilename || !strlen(infilename)) infilename = DEFAULT_INFILE;
    outfilename = outfileInput->text();
    if (!outfilename || !strlen(outfilename)) outfilename = DEFAULT_OUTFILE;
    statfilename = statfileInput->text();
    if (!statfilename || !strlen(statfilename))
	statfilename = DEFAULT_STATFILE;
    
    const char *blimey = altSizeInput->text();
    if (!blimey || !strlen(blimey)) altsize = DEFAULT_ALTSIZE;
    else altsize = atoi(blimey);

    paintarea = new PaintArea(MYAPP_NAME": Picture", this);
    paintarea->hide();

    connect(quitButton,  SIGNAL(clicked()), qApp,      SLOT(quit()));
    connect(splitButton, SIGNAL(clicked()), paintarea, SLOT(splitInfile()));
}


PrefDialog::~PrefDialog()
{
}

void
PrefDialog::saveVars() {

    infilename = infileInput->text();
    if (!infilename || !strlen(infilename)) infilename = DEFAULT_INFILE;
    outfilename = outfileInput->text();
    if (!outfilename || !strlen(outfilename)) outfilename = DEFAULT_OUTFILE;
    statfilename = statfileInput->text();
    if (!statfilename || !strlen(statfilename))
	statfilename = DEFAULT_STATFILE;

    const char *newX = newXInput->text();
    const char *newY = newYInput->text();
    const char *newSX = splitXInput->text();
    const char *newSY = splitYInput->text();
    const char *newAlt = altSizeInput->text();
	
    if (!newX || !strlen(newX)) tileSizeX=DEFAULT_TILEX;
    else tileSizeX=atoi(newX);
    
    if (!newY || !strlen(newY)) tileSizeY=DEFAULT_TILEY;
    else tileSizeY=atoi(newY);

    if (!newAlt || !strlen(newAlt)) altsize=DEFAULT_ALTSIZE;
    else altsize=atoi(newAlt);
	
    if (!newSX || !strlen(newSX)) splitX=PRINTABLE_WIDTH;
    else splitX=atoi(newSX);
    
    if (!newSY || !strlen(newSY)) splitY=PRINTABLE_HEIGHT;
    else splitY=atoi(newSY);

    
    origdisp = dispOrigCheck->isChecked();
    dcheck = darkTestCheck->isChecked();
    colsubst = colSubstCheck->isChecked();
    contdisp = contDispCheck->isChecked();
    pcheck = printableCheck->isChecked();
    
    paintarea->startProcess();
}

void
PrefDialog::getInfile() {
    
    QString f = QFileDialog::getOpenFileName( ".", "*.jpg");
    if ( !f.isEmpty() ) {
	infileInput->setText(f);
    } 
}

void
PrefDialog::getOutfile() {
    
    QString f = QFileDialog::getSaveFileName( ".", "*.bmp");
    if ( !f.isEmpty() ) {
	outfileInput->setText(f);
    } 

}

void
PrefDialog::getStatfile() {
    
    QString f = QFileDialog::getOpenFileName( ".", "*.dat");
    if ( !f.isEmpty() ) {
	statfileInput->setText(f);
    } 

}
