/*
 * Implementation of divisors and jpeg-related code
 * Copyright (c) 1998 - Stig Erik Sand
 *
 * $Id: divide.cpp,v 1.13 1998/04/28 12:33:47 stig Exp $
 */

#include "divide.H"
#include "picture.H"
#include <stdio.h>

extern "C" {
#include <jpeglib.h>
	   }

#include <string.h>
#include <stdlib.h>
#include <limits.h>
#include <iostream.h>


void
PictProc::dividePictures(const char **filenames, int numfiles) {

    SmallPicture *pics;
    int totalpics=0;
    int howmany;
 
    FILE *statFile = fopen(DEFAULT_STATFILE, "wb");
    fwrite(&totalpics, sizeof(int), 1, statFile);

    COUTPUT("Size of SmallPicture is: " << sizeof(SmallPicture));
    
    for (int i=0; i < numfiles; i++) {
	COUTPUT("Doing divide and split on(" << i << ") : " << filenames[i]);
	pics = Picture::dividePicture(filenames[i]);
	if (pics) {
//	    COUTPUT("Done divide");
	    howmany=2; // blah
	    fwrite(pics, sizeof(SmallPicture), howmany, statFile);
	    totalpics+=howmany;
	    delete[] pics;
	}
    }
    fseek(statFile, 0L, SEEK_SET); 
    fwrite(&totalpics, sizeof(int), 1, statFile);
    fclose(statFile);
}

SmallPicture *
PictProc::buildPictureTable(int &howmany, const char *statfilename) {

    FILE *statFile = fopen(statfilename, "rb");

    fread(&howmany, sizeof(int), 1, statFile);

    SmallPicture *pics = new SmallPicture[howmany+1];

    for (int i=0; i < howmany; i++) {
	fread(&pics[i], sizeof(SmallPicture), 1, statFile);
    }
    	
    fclose(statFile);
    
    return pics;
}

struct KeepAppUp {
    KeepAppUp() : index(-1), value(-1) { }
    int index, value;
};

static void
localSort(KeepAppUp *arr, int arrsize) {

    for (int i=0; i<arrsize; i++) {
	KeepAppUp tmp = arr[i];
	int j=i;
	for (; j>0 && tmp.value > arr[j-1].value; j--)
	    arr[j]=arr[j-1];
	arr[j]=tmp;
    }

}


int *
PictProc::findMatch(const SmallPicture *sm, int howmany,
		    const PicturePart *pp, int all, int dcheck,
		    int arrsize) {

    int *retarr = new int[arrsize];
    for (int j=0; j<arrsize; j++) retarr[j] = -1;

    KeepAppUp *values = new KeepAppUp[arrsize+1];
//    int highest=0;
    
    for (int i=0; i < howmany; i++) {
	values[arrsize].index = i;
	values[arrsize].value = pp->equalitySum(sm[i].data, all, dcheck);

	localSort(values, arrsize+1);
    }

//    cout << "[" << (values[0].value/100000) << ", "
//	 << (values[arrsize-1].value/100000) << "]" << endl;
    
    for (int k=0; k < arrsize; k++) {
	if (values[k].value>0)
	    retarr[k] = values[k].index;
	else {
	    retarr[k] = -1;
	}
    }


    delete[] values;
    
    cout.flush();
//    cout << "Score: " << highest << " ";
//    cout << "Returning index " << index << " " << highest << endl;
    return retarr;

}

QImage *
PictProc::getJpegImage(FILE *infile) {

    jpeg_decompress_struct cinfo;
    jpeg_error_mgr jerr;

    cinfo.err = jpeg_std_error(&jerr);
    jpeg_create_decompress(&cinfo);

    jpeg_stdio_src(&cinfo, infile);
    jpeg_read_header(&cinfo, TRUE);
  
//    cout << "Width: " << cinfo.image_width << " Height: "
//	 << cinfo.image_height <<  endl;
    
//    cinfo.out_color_space = JCS_GRAYSCALE;
//    cinfo.out_color_space = JCS_RGB;
    cinfo.quantize_colors = TRUE;

    if (cinfo.out_color_space==JCS_GRAYSCALE) return 0;
    
    jpeg_start_decompress(&cinfo);

    const int blah = cinfo.output_width*cinfo.output_components;
    QImage *pnt = Picture::initImage(cinfo.output_width, cinfo.output_height,
				     8, 256);
    JSAMPARRAY buffer = (*cinfo.mem->alloc_sarray)((j_common_ptr) &cinfo,
					JPOOL_IMAGE, blah, 1);   

/*
  cout << "Blah: " << blah << endl;
  cout << "comp: " << cinfo.out_color_components << " "
  << cinfo.output_components << " "
  << cinfo.actual_number_of_colors << " "
  << cinfo.data_precision << endl;
  
  printf("image color-space = %d\n", cinfo.jpeg_color_space);
  printf("image components = %d\n", cinfo.num_components);
  printf("output color-space = %d\n", cinfo.out_color_space);
*/
    Picture::setColours(pnt, cinfo.colormap, cinfo.actual_number_of_colors);


    for (int i=0; cinfo.output_scanline < cinfo.output_height; i++) {
	jpeg_read_scanlines(&cinfo, buffer, 1);
	Picture::copyBuffer(pnt, buffer[0], blah, i);
    }

    jpeg_finish_decompress(&cinfo);
    jpeg_destroy_decompress(&cinfo);
    fclose(infile);
    
    return pnt;
}

