/*
 * The implementation of the painting area
 * Copyright (c) 1998 - Stig Erik Sand
 *
 * $Id: paintarea.cpp,v 1.18 1998/05/23 20:30:50 stig Exp $
 */

#include "paintarea.H"
#include <qapp.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qimage.h>
#include <qpopmenu.h>
#include <qcursor.h>
#include <qmsgbox.h>
#include <stdlib.h>
#include <iostream.h>
#include "PrefDialog.h"
#include "shake.H"
#include "picture.H"
#include "divide.H"

PaintArea::PaintArea(const char* name, PrefDialog *pooh)
    : QWidget(0, name,  110592), thepic(NULL), prefs(pooh),
      small(NULL), image(NULL), pixmaps(NULL), len(0), thepixmap(NULL)
{
    setCaption( "Ze Mosaic" );

    pictlist = new QPopupMenu;
    
    popmen = new QPopupMenu;
    popmen->insertItem("Pictures", pictlist);
    popmen->insertItem("Save as printable parts", this,
		       SLOT(saveAsPrintable()));
    popmen->insertItem("Quit", qApp, SLOT(quit()));
    connect(pictlist, SIGNAL(activated(int)),
	    this, SLOT(pasteInAlternative(int)));
    
}

PaintArea::~PaintArea() {

    if (thepic) delete thepic;
    if (small) delete[] small;
    if (image) delete image;
    if (pixmaps) delete[] pixmaps;
    if (thepixmap) delete thepixmap;
    if (len) {
	for (unsigned int j=0; j<len; j++) delete[] picture[j];
	delete[] picture;
    }
}

void
PaintArea::displayPicture(QImage *im) {
    
    QPixmap pix;
    pix.convertFromImage(*im);
    QPainter paint(&pix);
    bitBlt(this, 0, 0, &pix);
}

void
PaintArea::displayPicture() {
    
    if (!thepic) return;

    const int origdisp = prefs->origdisp;
    const int contdisp = prefs->contdisp;
    const int subst = prefs->colsubst;
    const int dcheck = prefs->dcheck;

    const int altsize = prefs->altsize;
    
    parts = thepic->getParts(len);

//    COUTPUT("Len=" << len << " (" << prefs->tileSizeX << ", " << prefs->tileSizeY << ")");
    
    int howmany;
    COUTPUT("Building picture table..");
    small = PictProc::buildPictureTable(howmany, prefs->statfilename);

    pixmaps = new QPixmap[howmany];
    
    thepixmap = new QPixmap(thepic->width, thepic->height);
    QPainter paint(thepixmap);
    
    if (origdisp) {
	QImage * footwork = Picture::makeImage(prefs->infilename);
	QPixmap rabbit;
	rabbit.convertFromImage(*footwork);
	paint.drawPixmap(0,0 /*MY_BORDER, MY_BORDER*/, rabbit);
	delete footwork;
    }
	
    picture = new int*[len];

//    QPixmap foo;

    for (unsigned i=0; i < len; i++) {
	picture[i] = PictProc::findMatch(small, howmany, &parts[i],
					 BASIC_ALLOW, dcheck, altsize);
	int whatallow=BASIC_ALLOW+1;
	while (picture[i][0]==-1) {
	    cout << "* ";
	    picture[i]=PictProc::findMatch(small, howmany, &parts[i],
					   whatallow++, !subst, altsize);
	}

//	cout << " (" << picture[i][0] << "@" << howmany << "#"
//	     << parts[i].coords[0].x << "," <<  parts[i].coords[0].y << ") ";
	for (int n=0; n<altsize; n++) {
	    if (picture[i][n]>-1) {
		if (pixmaps[picture[i][n]].width()==0) {
		    pixmaps[picture[i][n]] = getPixmap(small[picture[i][n]]);
		}
	    }
	}
	
	paint.drawPixmap(parts[i].coords[0].x, parts[i].coords[0].y,
			 pixmaps[picture[i][0]], 0, 0,
			 prefs->tileSizeX, prefs->tileSizeY);
	
	
	if (contdisp) bitBlt(this, MY_BORDER, MY_BORDER, thepixmap); //slow
    }
    cout << endl;
    
    bitBlt(this, MY_BORDER, MY_BORDER, thepixmap);
    thepixmap->save(prefs->outfilename, "BMP");
    
    if (prefs->pcheck) saveAsPrintable();

    cout << "\nUsed pictures: " << usedPictures(howmany) << " of "
	 << howmany << " pictures\n";
    cout << endl;

}

void
PaintArea::displaySmallPicture(QPainter *paint, int x, int y,
			       const int *pic) {

    const int tileX = prefs->tileSizeX;
    const int tileY = prefs->tileSizeY;
//    cout << "(Drawing at:" << x << "," << y << ")";
    pixmaps[*pic] = getPixmap(small[*pic]);
    paint->drawPixmap(x, y, pixmaps[*pic], 0, 0, tileX, tileY);

}


void
PaintArea::mousePressEvent ( QMouseEvent *e ) {

    if (e->button()==RightButton) {
	pictlist->clear();
	QPoint p = mapFromGlobal(QCursor::pos());
	int x = p.x();
	int y = p.y();
	int w = x / prefs->tileSizeX;
	int h = y / prefs->tileSizeY;

	int row = thepic->width / prefs->tileSizeX;
	whichtile = row*h+w;

//	COUTPUT("("<<x<<","<<y<<") ["<<w<<","<<h<<"] "<<row<<"%"<<whichtile);
	
	/* dummy put in the right pictures based on tile */
	for (int i=0; i< prefs->altsize; i++) {
	    if (picture[whichtile][i]!=-1)
		pictlist->insertItem(pixmaps[picture[whichtile][i]],i);
	}
	popmen->popup(QCursor::pos());
    }
	
}

void
PaintArea::startProcess() {

    setCursor(waitCursor);
    const char *fname = prefs->infilename;

    image = Picture::makeImage(fname);
    
    if (thepic) delete thepic;
    thepic = new MosaicPicture("mypicture", image,
			       prefs->tileSizeX, prefs->tileSizeY);

    resize(thepic->width+MY_BORDER*2, thepic->height+MY_BORDER*2);
    show();
    
    displayPicture();
    
    setCursor(arrowCursor);
    
}

QPixmap
PaintArea::getPixmap(const SmallPicture& pic) {

    const int tileX = prefs->tileSizeX;
    const int tileY = prefs->tileSizeY;

    QImage *muse = Picture::makeImage(pic.name);
    if (pic.flipped)
	muse = Picture::makeFlip(muse);
    QPixmap pix;
    pix.convertFromImage(*muse);
    delete muse;
	
    COUTPUT("(Reading in: " << pic.name << ", " << pix.width() << ")");
    QWMatrix m;
    float scaleX = (float)tileX/(float)pic.width;
    float scaleY = (float)tileY/(float)pic.height;
    m.scale(scaleX, scaleY);

    QPixmap blah(pix.xForm(m));
    return blah;
}

void
PaintArea::pasteInAlternative(int i) {
//    COUTPUT("Pasted in " << i);
    
    QPainter paint(thepixmap);    
    paint.drawPixmap(parts[whichtile].coords[0].x,
		     parts[whichtile].coords[0].y, 
		     pixmaps[picture[whichtile][i]], 0, 0,
		     prefs->tileSizeX, prefs->tileSizeY);
    
	
    bitBlt(this, MY_BORDER, MY_BORDER, thepixmap);
    thepixmap->save(prefs->outfilename, DEFAULT_OUTFORMAT);

}

void
PaintArea::paintEvent ( QPaintEvent * ) {
    /* dumb implementation */
    if (thepixmap)
	bitBlt(this, MY_BORDER, MY_BORDER, thepixmap);

}

void
PaintArea::saveAsPrintable() {

    if (thepixmap) {
	setCursor(waitCursor);

	QImage im = thepixmap->convertToImage();
	QPixmap ya_pix;

	const int splitX=prefs->splitX;
	const int splitY=prefs->splitY;
		
	const int hei = im.height();
	const int wid = im.width();
	int right = wid / splitX;
	const int rrem = wid-right*splitX;
	if (rrem) right++;
	int down = hei / splitY;
	const int rdown = hei-down*splitY;
	if (rdown) down++;
//	COUTPUT("Wid: " << wid << " hei: " << hei); 
	QImage *tmp;
	char fnbuffer[128];

	for (int i=0; i < right; i++) {
	    for (int j=0; j< down; j++) {
		const int sizex = ((i+1)*splitX < wid) ? splitX : wid - i*splitX;
		const int sizey = ((j+1)*splitY < hei) ? splitY : hei - j*splitY;
		const int startx = i*splitX;
		const int starty = j*splitY;
//		COUTPUT("X: " << sizex << " Y: " << sizey);
//		COUTPUT("SX: " << startx << " SY: " << starty);
		tmp = new QImage(sizex, sizey, 8, 256);
		const int num = im.numColors();
		tmp->setNumColors(num);
		QRgb* pah = im.colorTable();
		QRgb* blah = tmp->colorTable();
		for (int m=0; m<num; m++)
		    *blah++ = *pah++;
			
		for (int k=0; k<sizey; k++) {
		    Picture::copyBuffer(tmp, im.scanLine(starty+k)+startx,
					sizex, k);
		}
		ya_pix.convertFromImage(*tmp);
		sprintf(fnbuffer,"%s.%d.%d.bmp", "partpic", i, j);
		ya_pix.save(fnbuffer, DEFAULT_OUTFORMAT);
		delete tmp;
	    }
	}
	setCursor(arrowCursor);
    }


}

int
PaintArea::usedPictures(int len) const {
    
    int count=0;
    for (int i=0; i < len; i++) if (pixmaps[i].width()) count++;
    return count;

}

void
PaintArea::splitInfile() {

    prefs->infilename = prefs->infileInput->text();
    if (!prefs->infilename || !strlen(prefs->infilename))
	prefs->infilename = DEFAULT_INFILE;
    
    char buffer[128];
    const char *infname = prefs->infilename;
    if (!strstr(infname, "bmp")) {
	sprintf(buffer, "%s %s\n", infname, "is not a bmp picture");
    }
    else {
	thepixmap = new QPixmap(infname, "BMP");
	saveAsPrintable();
	sprintf(buffer, "%s %s\n", infname, "has been split");
    }
    QMessageBox::information( 0, "Macbeth", buffer);
}
