#ifndef PICTURE_H
#define PICTURE_H

/*
 * Headerfile for the Picture-related classes
 * Copyright (c) 1998 - Stig Erik Sand
 *
 * $Id: picture.H,v 1.6 1998/04/25 10:23:45 stig Exp $
 */

#include "shake.H"

class RGBColour {
public:
    RGBColour(int a=-1, int b=-1, int c=-1) : red(a), green(b), blue(c) { }

    int howEqual(const RGBColour&) const;
    int darkLevel() const;
    int dark() const;
    int light() const;
    
private:
    int red, green, blue;
};

class Coord {
public:
    Coord(int a=-1, int b=-1) : x(a), y(b), colour(-1,-1,-1) { }
    int x, y;
    RGBColour colour;
};

#define COORD_NUMBER 100
#define DARK_NUMBER 9
#define BASIC_ALLOW 1
#define SECOND_ALLOW 2

class PicturePart {
    
public:
    PicturePart(Coord tl=Coord(), Coord br=Coord());
    void calculateDescr();
    int equalitySum(const PicturePart&, int all, int checkd=0) const;
    
    Coord coords[COORD_NUMBER];
private:
    int dark[DARK_NUMBER];
};

class SmallPicture {
public:
    char name[80];
    char flipped;
    PicturePart data;
    int width, height;
};

class QPainter;
class QImage;
class Picture {
public:
    static void setColour(Coord& pt, QImage *im);
    static void setColours(QImage *, unsigned char**, int len);
    static void copyBuffer(QImage *, unsigned char*, int len, int index);
    static void paintDot(QPainter *, int r, int g, int b, int, int);
    static QImage *initImage(int w, int h, int d, int c);
    static QImage *makeImage(const char *filename);
    static QImage *makeImage(gdImagePtr);
    static QImage *makeFlip(QImage *);
    static SmallPicture *dividePicture(const char *filename);
//    static SmallPicture *dividePicture(const char *filename, int &);
};

class ostream;

class MosaicPicture : public Picture {
    friend class PaintArea;
public:
    MosaicPicture(const char *, QImage *, int tX, int tY);
    ~MosaicPicture();
    
    const PicturePart* getParts(unsigned int &len) const {
	len=howmany;
	return parts;
    }
    void printParts(ostream &);
    const char * theName() const { return name; }
    
    int getWidth() const { return width; }
    int getHeight() const { return height; }
    
private:
    char name[80];
    PicturePart *parts;
    unsigned int howmany;
    int width, height;
};

#endif /* PICTURE_H */

