/*
 * Implementation of Picture-related classes/functions
 * Copyright (c) 1998 - Stig Erik Sand
 *
 * $Id: picture.cpp,v 1.11 1998/04/25 20:34:21 stig Exp $
 */

#include "picture.H"
#include "divide.H"
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <iostream.h>
#include <qpainter.h>
#include <qimage.h>
#include <qmsgbox.h>

#define COLOUR_LIMIT 50
#define COLOUR_LIMIT_SQ 2500

inline int max(int a, int b) {
    return (a>b) ? a : b;
}

inline int min(int a, int b) {
    return (a<b) ? a : b;
}

int
RGBColour::howEqual(const RGBColour& rb) const {
    
    int score=0;
    int diffred   = max(rb.red, red)     - min(rb.red, red);
    int diffgreen = max(rb.green, green) - min(rb.green, green);
    int diffblue  = max(rb.blue, blue)   - min(rb.blue, blue);
    if (diffred<COLOUR_LIMIT)
	score += COLOUR_LIMIT_SQ - (diffred   * diffred);
    if (diffgreen<COLOUR_LIMIT)
	score += COLOUR_LIMIT_SQ - (diffgreen * diffgreen);
    if (diffblue<COLOUR_LIMIT)
	score += COLOUR_LIMIT_SQ - (diffblue  * diffblue);

/*    if (abs(rb.red-red)< COLOUR_LIMIT)
	score += COLOUR_LIMIT - abs(rb.red-red);
    
    if (abs(rb.green-green)< COLOUR_LIMIT)
	score += COLOUR_LIMIT - abs(rb.green-green);

    if (abs(rb.blue-blue)< COLOUR_LIMIT)
	score += COLOUR_LIMIT - abs(rb.blue-blue);
*/
    return score;
}

int
RGBColour::dark() const {

    if (red<100&&blue<100&&green<100) return 1;
    return 0;
}

int
RGBColour::light() const {

    if (red>175&&blue>175&&green>175) return 1;
    return 0;
}

int
RGBColour::darkLevel() const {

    return (red+green+blue)/77;
    
}

PicturePart::PicturePart(Coord tl, Coord br) {

    int width  = br.x-tl.x;
    int height = br.y-tl.y;

    for (int k=0; k < DARK_NUMBER; k++) dark[k] = 0;
    
    const int gridsize = (int)sqrt(COORD_NUMBER);
    int howfar=0;
    for (int x=0; x < gridsize; x++)
	for (int y=0; y < gridsize; y++) {
	    coords[howfar]=Coord(tl.x + (width*x) / gridsize,
				 tl.y + (height*y) / gridsize);
	    
	    howfar++;
	}

}


void
PicturePart::calculateDescr() {

    const int gridsize = (int)sqrt(COORD_NUMBER);
    const int thirdsize = gridsize/3;
    int howfar=0;

//    cout << "Gr: " << gridsize << " third: " << thirdsize << endl;
//    cout << "Val: " << dark[0] << " " << dark[1] << endl;
    
    for (int x=0; x < gridsize; x++)
	for (int y=0; y < gridsize; y++) {

	    if (y < thirdsize) { // Row 1

		if (x < thirdsize)
		    dark[0]+=coords[howfar].colour.darkLevel();
		else if (x >= gridsize-thirdsize)
		    dark[2]+=coords[howfar].colour.darkLevel();
		else
		    dark[1]+=coords[howfar].colour.darkLevel();
	    }
	    else if (y >= gridsize-thirdsize) { // Row 3

		if (x < thirdsize)
		    dark[6]+=coords[howfar].colour.darkLevel();
		else if (x >= gridsize-thirdsize)
		    dark[8]+=coords[howfar].colour.darkLevel();
		else
		    dark[7]+=coords[howfar].colour.darkLevel();
	    }
	    else { // Row 2

		if (x < thirdsize)
		    dark[3]+=coords[howfar].colour.darkLevel();
		else if (x >= gridsize-thirdsize)
		    dark[5]+=coords[howfar].colour.darkLevel();
		else
		    dark[4]+=coords[howfar].colour.darkLevel();
	    }
	    howfar++;
	}
	    
    const int locsize = COORD_NUMBER/9;
//    cout << "duh: " << dark[0] << "-" << dark[1];
    for (int i=0; i < DARK_NUMBER; i++)
	dark[i] = dark[i]/locsize;
//    cout << "#" << locsize << "#" << dark[0] << "-" << dark[1] << endl;
}   


int
PicturePart::equalitySum(const PicturePart& other, int allowed,
			 int checkdlimit) const {


    if (checkdlimit) {

	if (dark[0]>30) strcpy(NULL, "hello world");
	
	for (int k=0; k<DARK_NUMBER; k++) {
//	    cout << "Compare |" << dark[k] << "-"
//		 << other.dark[k] << "|" << endl;
	    if ((dark[k] < (other.dark[k] - allowed)) ||
		(dark[k] > (other.dark[k] + allowed))) return 0;
	}
    }
  
    int tl=0, tr=0, bl=0, br=0;
    const int gridsize = (int)sqrt(COORD_NUMBER);
    const int halfsize = gridsize/2 - 1;
    int h=0;
    int tempsum;
    for (int x=0; x < gridsize; x++) {
	for (int y=0; y < gridsize; y++) {

	    tempsum = coords[h].colour.howEqual(other.coords[h].colour);
	    if (x <= halfsize && y <= halfsize) tl += tempsum;
	    else if (x > halfsize && y <= halfsize) tr += tempsum;
	    else if (x <= halfsize && y > halfsize) bl += tempsum;
	    else if (x > halfsize && y > halfsize) br += tempsum;
	    else cout << "2ehh.." << x << " " << y << endl;
	    
	    h++;
	}
    }

    int totalsum=tl+tr+bl+br;
/*    
    const int partnumber = COORD_NUMBER / 4;
    const int bonuslimit = partnumber*(COLOUR_LIMIT_SQ*2);
    const int bonus = partnumber * COLOUR_LIMIT_SQ;

//    cout << "Medium " << totalsum << endl;
    if (tl>bonuslimit) totalsum += bonus;
    if (tr>bonuslimit) totalsum += bonus;
    if (bl>bonuslimit) totalsum += bonus;
    if (br>bonuslimit) totalsum += bonus;
*/
//    cout << "Returning " << totalsum << endl;
    return totalsum;
}


MosaicPicture::MosaicPicture(const char *p, QImage *pt, int tX, int tY) {

    ::strcpy(name, p);

    width = pt->width();  
    height = pt->height();

    int howx = width / tX;
    int howy = height / tY;
     
    howmany = howx * howy;
//    cout << howmany << " " << howx << " " << howy
//	 <<  " " << width << " " << height << endl;
    
    parts = new PicturePart[howmany+1];
    
    int last=0;

    for (int i=0; i < howy; i++) {
	for (int j=0; j < howx; j++) {
	    
	    PicturePart foo(Coord(j*tX,i*tY),Coord((j+1)*tX-1,(i+1)*tY-1));
	    for (int k=0; k< COORD_NUMBER; k++)
		Picture::setColour(foo.coords[k], pt);
	    foo.calculateDescr();
	    parts[last++]=foo;
	}
    }
}

MosaicPicture::~MosaicPicture() {
    
    delete[] parts;
}


void
MosaicPicture::printParts(ostream &os) {

//    os << "Size: " << howmany << endl;
    
//    for (unsigned int i=0; i < howmany; i++)
//	os << "(" << parts[i].topleft.x << "," << parts[i].topleft.y << ")("
//	   << parts[i].botright.x << "," << parts[i].botright.y << "), ";
    os << endl;
    
}

void
Picture::setColour(Coord &pt, QImage *im) {

    int howfar=0;
    int re[5], gr[5], bl[5];
    QRgb temp(im->pixel(pt.x, pt.y));
    re[howfar] = qRed(temp);
    gr[howfar] = qGreen(temp);
    bl[howfar] = qBlue(temp);
    howfar++;
    
    if (im->valid(pt.x-1, pt.y)) {
	temp = im->pixel(pt.x-1, pt.y);
	re[howfar] = qRed(temp);
	gr[howfar] = qGreen(temp);
	bl[howfar] = qBlue(temp);
	howfar++;
    }
    
    if (im->valid(pt.x, pt.y-1)) {	
	temp = im->pixel(pt.x, pt.y-1);
	re[howfar] = qRed(temp);
	gr[howfar] = qGreen(temp);
	bl[howfar] = qBlue(temp);
	howfar++;
    }
    
    if (im->valid(pt.x+1, pt.y)) {
	temp = im->pixel(pt.x+1, pt.y);
	re[howfar] = qRed(temp);
	gr[howfar] = qGreen(temp);
	bl[howfar] = qBlue(temp);
	howfar++;
    }
    
    if (im->valid(pt.x, pt.y+1)) {	
	temp = im->pixel(pt.x, pt.y+1);
	re[howfar] = qRed(temp);
	gr[howfar] = qGreen(temp);
	bl[howfar] = qBlue(temp);
	howfar++;
    }
    
    int avgred=0, avggreen=0, avgblue=0;
    for (int i=0; i<howfar; i++){
	avgred   += re[i];
    	avggreen += gr[i];
	avgblue  += bl[i];
    }
    
    avgred   /= howfar;
    avggreen /= howfar;
    avgblue  /= howfar;
    
    pt.colour = RGBColour(avgred, avggreen, avgblue);
/*    cout << "x(" << x  << ") y(" << y << ") col(" << temp << ") red("
	 << col.red << ") green(" << col.green
	 << ") blue(" << col.blue << ")" << endl;*/
}
	
void
Picture::paintDot(QPainter *pnt, int r, int g, int b, int x, int y) {

    if (g!=-1) {
	pnt->setPen(QColor(r, g, b));
	pnt->drawPoint(x,y);
    }
    else {
	pnt->setPen(QColor(r, r, r));
	pnt->drawPoint(x,y);
    }

}

void
Picture::setColours(QImage * im, unsigned char** arr, int len) {
    
    QRgb *pah = im->colorTable();
    for (int i=0; i< len; i++)
	*pah++ = qRgb(arr[0][i], arr[1][i], arr[2][i]);
   
}

void
Picture::copyBuffer(QImage *im, unsigned char* arr, int len, int index) {

    unsigned char *pah=im->scanLine(index);
    memcpy(pah, arr, len);

}

QImage *
Picture::initImage(int w, int h, int d, int c) {

    QImage * ni = new QImage(w,h,d,c);
    return ni;
}

QImage *
Picture::makeImage(const char *fname) {

    QImage *image;
    FILE *infile = fopen(fname, "rb");
    if (!infile) {
	char buffer[128];
	sprintf(buffer, "%s %s\n\n%s", "Could not find file ",
		fname, "Macbeth will now exit.");
	QMessageBox::critical( 0, "Macbeth", buffer);
	exit(-1);
    }
    const char *whatFormat = strstr(fname, "gif");
    if (whatFormat) {
	gdImagePtr imPtr = gdImageCreateFromGif(infile);
	image = makeImage(imPtr);
	gdImageDestroy(imPtr);
    }
    else if ((whatFormat = strstr(fname, "jpg"))) {
	image = PictProc::getJpegImage(infile);
    }
    
    return image;
}


SmallPicture *
Picture::dividePicture(const char *filename) {

//    COUTPUT("Making image (" << filename << ")");
    QImage *imPtr = Picture::makeImage(filename);

    if (!imPtr) return 0;
    
    const int howmany = 2;
//    COUTPUT("Using new..");
    SmallPicture *tmp = new SmallPicture[howmany];
    
    tmp[0].flipped=0;
    tmp[1].flipped=1;
    
    for (int i=0; i<howmany; i++) {
	tmp[i].width = imPtr->width();
	tmp[i].height = imPtr->height();
	::strcpy(tmp[i].name, filename);
    }

    int k;
    // normal
    tmp[0].data=PicturePart(Coord(0,0), Coord(tmp[0].width-1,
					      tmp[0].height-1));
    
    for (k=0; k < COORD_NUMBER; k++){
	setColour(tmp[0].data.coords[k],  imPtr);
    }

//    COUTPUT("Starts flip");
    imPtr = Picture::makeFlip(imPtr);

    // flipped
    tmp[1].data=PicturePart(Coord(0,0), Coord(tmp[1].width-1,
					      tmp[1].height-1));
    
    for (k=0; k < COORD_NUMBER; k++){
	setColour(tmp[1].data.coords[k],  imPtr);
    }
	
//    COUTPUT("Calcing descr");
    for (int j=0; j<howmany; j++) {
	tmp[j].data.calculateDescr();
    }
    
    delete imPtr;
    
    return tmp;
}

/*
SmallPicture *
Picture::dividePicture(const char *filename, int &howmany) {
    
    QImage *imPtr = makeImage(filename);

    int width = imPtr->width();
    int height = imPtr->height();

    const int RES_XSIZE=40;
    const int RES_YSIZE=40;
    
    int howx = width / RES_XSIZE;
    int howy = height / RES_YSIZE;

    howmany = howx * howy;
  
//    FILE *outfile;
SmallPicture *tmp = new SmallPicture[howmany+1];
*/
    /*

    QImage *newpic = new QImage(RES_XSIZE, RES_YSIZE, 8, 256);
    int howfar=0;
    for (int i=0; i < howy; i++) {
      for (int j=0; j < howx; j++) {
	    sprintf(tmp[howfar].name, "res/%s%d.%d", filename, i, j);
	    gdImageCopy(newpic, imPtr, 0, 0, j*RES_XSIZE, i*RES_YSIZE,
			RES_XSIZE, RES_YSIZE);
	    
	    tmp[howfar].data=PicturePart(Coord(0,0),
					 Coord(RES_XSIZE-1, RES_YSIZE-1));
	    
	    for (int k=0; k < COORD_NUMBER; k++){
		Picture::setColour(tmp[howfar].data.coords[k],  newpic);
	    }
	    tmp[howfar].data.calculateDescr();
	    outfile=fopen(tmp[howfar].name, "wb");
	    gdImageGif(newpic, outfile);
	    fclose(outfile);
	    howfar++;
	}
    }
    delete newpic;
    */
/*
    delete imPtr;

    
    return tmp;
}
*/
QImage *
Picture::makeImage(gdImagePtr imp) {
    
    QImage * newimg = new QImage(imp->sx, imp->sy, 8, 256);
    
    QRgb *foo = newimg->colorTable();
    for (int k=0; k< imp->colorsTotal; k++)
	*foo++ = qRgb(imp->red[k], imp->green[k], imp->blue[k]);

    unsigned char *pah;
    for (int i=0; i<imp->sy; i++) {
	pah=newimg->scanLine(i);
	for (int j=0; j<imp->sx; j++)
	    pah[j] = imp->pixels[j][i];
    }
    
    return newimg;
}

QImage *
Picture::makeFlip(QImage *im) {
    
    int hei = im->height();
    int wid = im->width();
    
    unsigned char *blah = new unsigned char[wid];
//    COUTPUT("Allocated " << wid+1 << " bytes");
    for (int i=0; i < hei; i++) {
	unsigned char *pah=im->scanLine(i);
	memcpy(blah, pah, wid);
	for (int j=0; j<wid; j++)
	    pah[j]=blah[wid-j-1];
    }
    
    delete[] blah;
    return im;
}

